import os.path
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))


def logDict(name):
    """
    This function provides a default logging configuration dictionary. There are two logging handlers. The console
    handler enables logging to stdout. The file handler eneables logging to files. The log files are rotating by size.

    :param name: name of log file
    :return: dictionary with logging configuration
    """
    return {
        'version': 1,
        'disable_existing_loggers': True,
        'formatters': {
            'verbose': {
                'format': '[%(asctime)s] Got %(levelname)s in module %(module)s in function %(funcName)s in line %(lineno)s of %(pathname)s: %(message)s'
            },
            'simple': {
                'format': '%(levelname)s %(message)s'
            },
        },
        'filters': {
            'special': {
                'foo': 'bar',
            }
        },
        'handlers': {
            'console': {
                'level': 'INFO',
                'class': 'logging.StreamHandler',
                'formatter': 'verbose',
            },
            'file': {
                'class': 'logging.handlers.RotatingFileHandler',
                'level': 'INFO',
                'formatter': 'verbose',
                'filename': name + '.log',
                'mode': 'a',
                'maxBytes': 1024 * 1024 * 100,
                'backupCount': 5,
            }
        },
        'loggers': {
            '': {
                'handlers': ['console', 'file'],
                'level': 'INFO',
                'propagate': True,
            }
        }
    }