#!/usr/bin/env python
"""
Module name:		
callNEDM.py
Description:	
	calling routine Neustrelitz 3D Electron Density Model 
Needs:		python 2.6.6	
Version:	v1
Author:		Mainul Hoque
Created:	27.11.2014
Copyright:	(c) Mainul Hoque 2014
Mail:		Mainul.Hoque@dlr.de
Licence:	DLR
#-------------------------------------------------------------------------------
"""
import numpy as np
import pylab as pl
import bin.reconstruct.ModelNPDM_NPHM.ionofunclib as myfunc
from bin.reconstruct.ModelNPDM_NPHM import NEDMv1 as NEDM


def main():
	
	"""
	Testing each class / method
	"""
	
	ionparmodelObj = NEDM.ionpar()
	print(ionparmodelObj)
	
	#initialization
	lat_deg = np.array([10, 70],float)                                  #WGS84 geographic latitude Vector in deg 
	lon_deg = np.array([45, 50],float)                                  #WGS84 geographic longitude Vector in deg
	UT_hrs = np.array([10, 15],float)                                   #Universal Time (UT) in hours Vector
	doy = np.array([10, 15],float)                                      #day of year Vector
	F10p7_sfu = np.array([80, 100],float) 	                            #solar radio flux in flux units Vector
	altmRx = np.array([1., 1.],float)                                   #receiver altitude Vector in meters
	altmTx = np.array([20200000, 20200000],float)                       #transmitter altitude Vector in meters
	
	#get 2D model estimates
	modelObj = NEDM.model( lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu )   #call class object 
	print "TEC [TECU]:", modelObj.NTCM()                                #print individual model parameters
	print "NmF2 [1.e+12 el/m3]:", modelObj.NPDM()
	print "hmF2 [km]:", modelObj.NPHM()
	print "Np0 [1.e+12 el/m3]:", modelObj.Np0()
	print "Hplasma [km]:", modelObj.Hplasma()
	
	#get electron density distribution and resultant slant TEC along a line connecting P1 and P2 
	x1, y1, z1 = myfunc.geodetic2ecef(np.deg2rad(lat_deg), np.deg2rad(lon_deg), altmRx)    #P1 coordinate
	x2, y2, z2 = myfunc.geodetic2ecef(np.deg2rad(lat_deg), np.deg2rad(lon_deg), altmTx)    #P2 coordinate
	parset 	= {	"IonLimitm" : 1000000.,	"IonStepm" : 20000.,	"PlasmaStepm" : 50000.}    #iono parameter set optinal
	stecObj = NEDM.stec( x1[0], y1[0], z1[0], x2[1], y2[1], z2[1],UT_hrs[0], doy[0], F10p7_sfu[0], parset['IonLimitm'], parset['IonStepm'],	parset['PlasmaStepm'])
	print "TEC [TECU]:", stecObj.stec()
	
	#plot electron model versus altituide
	Ne, altkm = stecObj.getNe()
	pl.plot(Ne, altkm)
	pl.xlabel('Electron Density [1.e+12 el/$m^3$]')
	pl.ylabel("Altitude [km]")
	pl.ylim(0, 1000)	
	pl.show()
	
	
if __name__ == "__main__":
    main()
	

