# coding=utf-8
import datetime
import NeQuick.NeQuick_PythonEnvironment.NeQuickWrapperVectorInput as NeQuick
import module.merge_data as merge
from mpl_toolkits.basemap import Basemap
import numpy as np
import matplotlib.pyplot as plt
import bin.reconstruct.ModelNPDM_NPHM.NEDMv1 as NEDM
import Filter.config as FilterConfig
import KPFileReader.KPFileReader as KPReader
import MeasurementStations as MS
import OrdinaryKriging as myOK
import KrigeValidation as KrigeVal
import config
import os
import matplotlib
import Filter.filter_option4 as Filter
import matplotlib.colors as mc

class calcReconstruction(object):
    """
    This class does:
        - reconstructIono the hmF2 and NmF2 parameters of the F2-Layer.
        Not finished yet
    """

    def __init__(self):
        '''
        Konstructor. It reads and validates all parameter from the config file. Additionally some parameters are initialized.
        :return:
        '''

        self.Layer = config.layer
        if "hmF2" not in self.Layer and "NmF2" not in self.Layer and "foF2" not in self.Layer:
            self.writeInLog("ERROR: Possible layer input: hmF2, NmF2 and hmF2NmF2 or NmF2hmF2")
            raise AssertionError("Possible layer input: hmF2, NmF2 and hmF2NmF2 or NmF2hmF2")

        self.DoValidation = config.validationfile
        self.Resolution = config.resolution
        self.PathToCompFileNmF2 = config.pathToValidationFileNmF2
        self.PathToCompFilehmF2 = config.pathToValidationFilehmF2
        self.RefStations = config.refStations

        try: bool(self.DoValidation)
        except:
            self.writeInLog("The 'validationfile' variable is not a boolean value!")
            raise AssertionError("The 'validationfile' variable is not a boolean value!")

        try: self.Resolution = float(self.Resolution)
        except:
            self.writeInLog("Resolution is not a number!")
            raise AssertionError("Resolution is not a number!")

        self.Outputfolder = config.outputfolder
        try:
            if not os.path.isdir(self.Outputfolder):
                os.makedirs(self.Outputfolder)
        except:
            self.writeInLog("ERROR: Could not create directory: " + self.Outputfolder)
            raise AssertionError("Could not create directory: " + self.Outputfolder)

        self.OutputfolderLog = config.outputfolderLogs
        try:
            if not os.path.isdir(self.OutputfolderLog):
                os.makedirs(self.OutputfolderLog)
        except:
            self.writeInLog("ERROR: Could not create directory: " + self.OutputfolderLog)
            raise AssertionError("Could not create directory: " + self.OutputfolderLog)


        acdate = datetime.date.today()
        self.LogFileName = "Logfile_" + str(acdate) + ".txt"
        try:
            logfile = open(self.OutputfolderLog + self.LogFileName, "w")
            logfile.close()
        except:
            self.writeInLog("Could not create Log file")
            raise AssertionError("Could not create Log file")

        if self.DoValidation:
            if 180 % self.Resolution != 0:
                self.writeInLog("Resulition is not multiple 180!")
                raise AssertionError("Resulition is not multiple 180!")

            if not self.RefStations:
                self.writeInLog("No reference station mentioned!")
                raise AssertionError("No reference station mentioned!")

        try:
            logfile = open(self.OutputfolderLog + "ListOfResults.dat", "w")
            logfile.close()
        except:
            self.writeInLog("Could not create 'ListOfResults.dat' file")
            raise AssertionError("Could not create 'ListOfResults.dat' file")

        self.writeInListofResults(self.LogFileName)

        self.writeInLog("---------------------------------------\n---------------------------------------\n" +
                        str(datetime.datetime.now()) + "\nReading config file...\n")

        self.Time = config.time
        try: self.TimeIntervall = int(config.timeintervall)
        except:
            self.writeInLog("ERROR: Timeinterval is not a number!")
            raise AssertionError("Timeinterval is not a number!")

        try: self.Startdoy = int(config.startdoy)
        except:
            self.writeInLog("ERROR: Startdoy is not a number!")
            raise AssertionError("Startdoy is not a number!")

        try: self.Enddoy = int(config.enddoy)
        except:
            self.writeInLog("ERROR: Enddoy is not a number!")
            raise AssertionError("Enddoy is not a number!")


        self.PathSpidr_hmF2 = config.pathspidr_hmF2
        self.PathDataAll_hmF2 = config.pathdata_all_hmF2
        self.PathDidb_hmF2 = config.pathdidb_hmF2
        self.PathNgdc_hmF2 = config.pathngdc_hmF2

        self.PathSpidr_NmF2 = config.pathspidr_NmF2
        self.PathDataAll_NmF2 = config.pathdata_all_NmF2
        self.PathDidb_NmF2 = config.pathdidb_NmF2
        self.PathNgdc_NmF2 = config.pathngdc_NmF2

        try: self.LagDistance = int(config.lagDistance)
        except:
            self.writeInLog("ERROR: Lag distance is not a number!")
            raise AssertionError("Lag distance is not a number!")

        self.PathF107 = config.f107files
        self.VarModel = config.variogrammodel

        if self.LagDistance < 1:
            self.writeInLog("ERROR: Lag distance is < 1")
            raise AssertionError("Lag distance is < 1")

        if not len(self.Time) == 5:
            self.writeInLog("ERROR: Check the format for the time variable (HH:MM)!")
            raise AssertionError("Check the format for the time variable (HH:MM)!")

        try:
            hour = int(self.Time[:-3])
            minutes = int(self.Time[3:])
        except:
            self.writeInLog("ERROR: Could not convert hour and minutes of time in an integer value. Check format (HH:MM)")
            raise AssertionError("Could not convert hour and minutes of time in an integer value. Check format (HH:MM)")

        if not 0 <= hour < 24:
            self.writeInLog("ERROR: The hour of the time is not in the value range (0 <= hour < 24)!")
            raise AssertionError("The hour of the time is not in the value range (0 <= hour < 24)!")
        if not 0 <= minutes < 60:
            self.writeInLog("ERROR: The minute of the time is not in the value range (0 <= hour < 60)!")
            raise AssertionError("The minute of the time is not in the value range (0 <= hour < 60)!")

        self.BackModel = config.backgroundmodel.lower()
        if not "nequick" in self.BackModel and not "nphm" in self.BackModel and not "npdm" in self.BackModel:
            raise AssertionError("The backgroundmodel is neither NeQuick or NPHM nor NPDM!")
        if self.BackModel == "nphm" and "NmF2" in self.Layer:
            raise AssertionError("Wrong model (NPHM) for the layer NmF2!")
        if self.BackModel == "npdm" and "hmF2" in self.Layer:
            raise AssertionError("Wrong model (NPDM) for the layer hmF2!")

        self.Hour = hour + self.convertMinInHour(minutes)

        if self.TimeIntervall < 0:
            self.writeInLog("ERROR: Timeintervall is < 0")
            raise AssertionError("Timeintervall is < 0")

        if not os.path.isdir(self.PathSpidr_hmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathSpidr_hmF2)
            raise AssertionError("Could not find directory: " + self.PathSpidr_hmF2)
        if not os.path.isdir(self.PathDataAll_hmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathDataAll_hmF2)
            raise AssertionError("Could not find directory: " + self.PathDataAll_hmF2)
        if not os.path.isdir(self.PathDidb_hmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathDidb_hmF2)
            raise AssertionError("Could not find directory: " + self.PathDidb_hmF2)
        if not os.path.isdir(self.PathNgdc_hmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathNgdc_hmF2)
            raise AssertionError("Could not find directory: " + self.PathNgdc_hmF2)

        if not os.path.isdir(self.PathSpidr_NmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathSpidr_NmF2)
            raise AssertionError("Could not find directory: " + self.PathSpidr_NmF2)
        if not os.path.isdir(self.PathDataAll_NmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathDataAll_NmF2)
            raise AssertionError("Could not find directory: " + self.PathDataAll_NmF2)
        if not os.path.isdir(self.PathDidb_NmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathDidb_NmF2)
            raise AssertionError("Could not find directory: " + self.PathDidb_NmF2)
        if not os.path.isdir(self.PathNgdc_NmF2):
            self.writeInLog("ERROR: Could not find directory: " + self.PathNgdc_NmF2)
            raise AssertionError("Could not find directory: " + self.PathNgdc_NmF2)

        if not config.isotropyAnisotropy == "isotropy" and not config.isotropyAnisotropy == "anisotropy":
            self.writeInLog("ERROR: Choose isotropy or anisotropy")
            raise AssertionError("ERROR: Choose isotropy or anisotropy")

        if not os.path.isdir(self.PathF107):
            self.writeInLog("ERROR: Could not find directory: " + self.PathF107)
            raise AssertionError("Could not find directory: " + self.PathF107)

        if self.Startdoy > self.Enddoy:
            self.writeInLog("ERROR: Startdoy is higher than enddoy!")
            raise AssertionError("Startdoy is higher than enddoy!")

        if self.Startdoy <= 0 or self.Startdoy > 365:
            self.writeInLog("ERROR: Startdoy is not in the intervall 1-365!")
            raise AssertionError("ERROR: Startdoy is not in the intervall 1-365!")

        if self.Enddoy <= 0 or self.Enddoy > 365:
            self.writeInLog("ERROR: Enddoy is not in the intervall 1-365!")
            raise AssertionError("ERROR: Enddoy is not in the intervall 1-365!")

        if "exp" not in self.VarModel and "sph" not in self.VarModel and "gau" not in self.VarModel and "pow" not in self.VarModel:
            self.writeInLog("ERROR: Possibilities for the variogram model are:\n'sph' for spherical\n'exp' for exponential\n'gau' for gaussian\npow for power")
            raise AssertionError("Possibilities for the variogram model are:\n'sph' for spherical\n'exp' for exponential\n'gau' for gaussian\npow for power")

        #initialise some values
        self.AltList = range(1, 600, 1)

        self.F107 = []
        self.Year = 0

        self.writeInLog("Finished reading config file\n")

    def startAllParameter(self):
        '''
        This function is only used, when both parameters (hmF2 and NmF2) are named in the configfile. It starts the "runReconstruction"
        routine for both parameters.
        :return:
        '''

        #param hmF2
        self.Layer = "hmF2"
        self.runReconstruction()

        #Param NmF2
        self.Layer = "NmF2"
        self.AllStations = MS.Measurementstations(True, self.Layer, refstations=self.RefStations)
        self.UniqueStations = MS.Measurementstations(False, self.Layer, refstations=self.RefStations)
        self.F107 = []
        self.runReconstruction()

    def createValFiles(self):
        refFilename = "Error at reference File"
        try:
            if "nphm" in self.BackModel:
                if "sph" in self.VarModel:
                    refFilename = self.OutputfolderLog + self.Layer + "_nphmsphResidualsRefRel.dat"
                    self.generateValidationFile(refFilename)
                    refFilename = self.OutputfolderLog + self.Layer + "_nphmsphResidualsRefAbs.dat"
                    self.generateValidationFile(refFilename)
                if "exp" in self.VarModel:
                    refFilename = self.OutputfolderLog + self.Layer + "_nphmexpResidualsRefRel.dat"
                    self.generateValidationFile(refFilename)
                    refFilename = self.OutputfolderLog + self.Layer + "_nphmexpResidualsRefAbs.dat"
                    self.generateValidationFile(refFilename)
                if "gau" in self.VarModel:
                    refFilename = self.OutputfolderLog + self.Layer + "_nphmgauResidualsRefRel.dat"
                    self.generateValidationFile(refFilename)
                    refFilename = self.OutputfolderLog + self.Layer + "_nphmgauResidualsRefAbs.dat"
                    self.generateValidationFile(refFilename)
                if "pow" in self.VarModel:
                    refFilename = self.OutputfolderLog + self.Layer + "_nphmpowResidualsRefRel.dat"
                    self.generateValidationFile(refFilename)
                    refFilename = self.OutputfolderLog + self.Layer + "_nphmpowResidualsRefAbs.dat"
                    self.generateValidationFile(refFilename)
            if "npdm" in self.BackModel:
                if "sph" in self.VarModel:
                    refFilename = self.OutputfolderLog + self.Layer + "_npdmsphResidualsRefRel.dat"
                    self.generateValidationFile(refFilename)
                    refFilename = self.OutputfolderLog + self.Layer + "_npdmsphResidualsRefAbs.dat"
                    self.generateValidationFile(refFilename)
                if "exp" in self.VarModel:
                    refFilename = self.OutputfolderLog + self.Layer + "_npdmexpResidualsRefRel.dat"
                    self.generateValidationFile(refFilename)
                    refFilename = self.OutputfolderLog + self.Layer + "_npdmexpResidualsRefAbs.dat"
                    self.generateValidationFile(refFilename)
                if "gau" in self.VarModel:
                    refFilename = self.OutputfolderLog + self.Layer + "_npdmgauResidualsRefRel.dat"
                    self.generateValidationFile(refFilename)
                    refFilename = self.OutputfolderLog + self.Layer + "_npdmgauResidualsRefAbs.dat"
                    self.generateValidationFile(refFilename)
                if "pow" in self.VarModel:
                    refFilename = self.OutputfolderLog + self.Layer + "_npdmpowResidualsRefRel.dat"
                    self.generateValidationFile(refFilename)
                    refFilename = self.OutputfolderLog + self.Layer + "_npdmpowResidualsRefAbs.dat"
                    self.generateValidationFile(refFilename)
            if "nequick" in self.BackModel:
                if "sph" in self.VarModel:
                    refFilename = self.OutputfolderLog + self.Layer + "_nequicksphResidualsRefRel.dat"
                    self.generateValidationFile(refFilename)
                    refFilename = self.OutputfolderLog + self.Layer + "_nequicksphResidualsRefAbs.dat"
                    self.generateValidationFile(refFilename)
                if "exp" in self.VarModel:
                    refFilename = self.OutputfolderLog + self.Layer + "_nequickexpResidualsRefRel.dat"
                    self.generateValidationFile(refFilename)
                    refFilename = self.OutputfolderLog + self.Layer + "_nequickexpResidualsRefAbs.dat"
                    self.generateValidationFile(refFilename)
                if "gau" in self.VarModel:
                    refFilename = self.OutputfolderLog + self.Layer + "_nequickgauResidualsRefRel.dat"
                    self.generateValidationFile(refFilename)
                    refFilename = self.OutputfolderLog + self.Layer + "_nequickgauResidualsRefAbs.dat"
                    self.generateValidationFile(refFilename)
                if "pow" in self.VarModel:
                    refFilename = self.OutputfolderLog + self.Layer + "_nequickpowResidualsRefRel.dat"
                    self.generateValidationFile(refFilename)
                    refFilename = self.OutputfolderLog + self.Layer + "_nequickpowResidualsRefAbs.dat"
                    self.generateValidationFile(refFilename)
        except:
            self.writeInLog("Could not create " + refFilename)
            raise AssertionError("Could not create " + refFilename)

    def runReconstruction(self):
        '''
        This is the main routine to start with. It managed all other functions.
        :return:
        '''


        if "hmF2" in self.Layer and ("NmF2" in self.Layer or "foF2" in self.Layer):
            self.startAllParameter()
            return

        if self.DoValidation: self.createValFiles()


        if "hmF2" in self.Layer:
            self.PathToCompFile = self.PathToCompFilehmF2
        elif "NmF2" in self.Layer:
            self.PathToCompFile = self.PathToCompFileNmF2
        else:
            self.writeInLog("Layer Error! Layer is neither hmF2 nor NmF2!")
            raise AssertionError("Layer Error! Layer is neither hmF2 nor NmF2!")

        self.AllStations = MS.Measurementstations(True, self.Layer, refstations=self.RefStations)
        self.UniqueStations = MS.Measurementstations(False, self.Layer, refstations=self.RefStations)

        #self.writeInLog("--------------------\nStarting calculation for: " + self.Layer + "\n--------------------")

        ###############################################################
        ############Get Data from every Station########################
        ###############################################################
        self.writeInLog("Start reading data files...")
        self.readData()
        self.writeInLog("Finished reading data files")

        if self.UniqueStations.getCount() == 0: return

        ###############################################################
        ############Get f10.7 values for every day#####################
        ###############################################################
        self.writeInLog("Start reading F10.7 values...")
        self.getf107Values()
        self.writeInLog("Finished reading F10.7 values")


        ###############################################################
        ################Plot interpolation area########################
        ###############################################################
        self.writeInLog("Start plotting interpolation area with measurement stations...")
        self.plotStations()
        self.writeInLog("Finished plotting interpolation area with measurement stations")


        ###############################################################
        ##########Plot value with respect to Longitude#################
        ###############################################################
        self.writeInLog("Start plotting values with respect to longitude...")
        self.plotValueWithRespectToCoords(self.UniqueStations.getAllLon(), self.AllStations.getAllLon(), "Longitude")
        self.writeInLog("Finished plotting values with respect to longitude")

        ###############################################################
        ###########Plot value with respect to Latitude#################
        ###############################################################
        self.writeInLog("Start plotting values with respect to latitude...")
        self.plotValueWithRespectToCoords(self.UniqueStations.getAllLat(), self.AllStations.getAllLat(), "Latitude")
        self.writeInLog("Finished plotting values with respect to longitude")

        ###############################################################
        #####################Plot Histogram############################
        ###############################################################
        self.writeInLog("Start plotting histogram...")
        self.plotHistogram()
        self.writeInLog("Finished plotting histogram")

        ###############################################################
        ############Plot NeQuick Model for one station#################
        ###############################################################
        self.writeInLog("Start plotting NeQuick model for one station...")
        if "nequick" in self.BackModel: self.plotNeQuickOneStation()
        self.writeInLog("Finished plotting NeQuick model for one station")

        ###############################################################
        ###############Plot NPDM Model for one station#################
        ###############################################################
        self.writeInLog("Start plotting NPDM/NPHM model for one station...")
        if "npdm" in self.BackModel or "nphm" in self.BackModel: self.plotNPDMorNPHMOneStation()
        self.writeInLog("Finished plotting NPDM/NPHM model for one station")

        ###############################################################
        #######Plot Data with respect to NeQuick Model#################
        ###############################################################
        self.writeInLog("Start plotting relative difference between NeQuick model and measurement...")
        if "nequick" in self.BackModel: diffMeasurementNeQuick = self.plotNeQuickDiffMeasurement()
        self.writeInLog("Finished plotting relative difference between NeQuick model and measurement")

        ###############################################################
        ##########Get Data from the Model NPDM/NPHM####################
        ###############################################################
        self.writeInLog("Start plotting relative difference between NPDM/NPHM model and measurement...")
        if "npdm" in self.BackModel or "nphm" in self.BackModel: diffNPDMorNPHM = self.plotNPDMDorNPHMDiffMeasurement()
        self.writeInLog("Finished plotting relative difference between NPDM/NPHM model and measurement")

        ###############################################################
        ################Kriging (Model: NeQick)########################
        ###############################################################
        if "nequick" in self.BackModel:
            self.UniqueStations.addResiduals(diffMeasurementNeQuick)
            if "sph" in self.VarModel:
                self.writeInLog("Start Kriging; NeQuick; spherical variogram model...")
                self.doOK("nequick", variogrammodel="sph")
                self.writeInLog("Finished Kriging; NeQuick; spherical variogram model")

            if "exp" in self.VarModel:
                self.writeInLog("Start Kriging; NeQuick; exponential variogram model...")
                self.doOK("nequick", variogrammodel="exp")
                self.writeInLog("Finished Kriging; NeQuick; exponential variogram model")

            if "gau" in self.VarModel:
                self.writeInLog("Start Kriging; NeQuick; gaussian variogram model...")
                self.doOK("nequick", variogrammodel="gau")
                self.writeInLog("Finished Kriging; NeQuick; gaussian variogram model")

            if "pow" in self.VarModel:
                self.writeInLog("Start Kriging; NeQuick; power variogram model...")
                self.doOK("nequick", variogrammodel="pow")
                self.writeInLog("Finished Kriging; NeQuick; power variogram model")

        ###############################################################
        ################Kriging (Model: NPDM/NPHM)#####################
        ###############################################################
        if "npdm" in self.BackModel or "nphm" in self.BackModel:
            self.UniqueStations.addResiduals(diffNPDMorNPHM)
            if "sph" in self.VarModel:
                if self.Layer == "foF2" or self.Layer == "NmF2":
                    self.writeInLog("Start Kriging; NPDM; spherical variogram model...")
                    self.doOK("npdm", variogrammodel="sph")
                    self.writeInLog("Finished Kriging; NPDM; spherical variogram model")
                elif self.Layer == "hmF2":
                    self.writeInLog("Start Kriging; NPHM; spherical variogram model...")
                    self.doOK("nphm", variogrammodel="sph")
                    self.writeInLog("Finished Kriging; NPHM; spherical variogram model")
                else:
                    self.writeInLog("ERROR: Could not find Layer: " + self.Layer)
                    raise AssertionError("Layer nicht gefunden")

            if "exp" in self.VarModel:
                if self.Layer == "foF2" or self.Layer == "NmF2":
                    self.writeInLog("Start Kriging; NPDM; exponential variogram model...")
                    self.doOK("npdm", variogrammodel="exp")
                    self.writeInLog("Finished Kriging; NPDM; exponential variogram model")
                elif self.Layer == "hmF2":
                    self.writeInLog("Start Kriging; NPHM; exponential variogram model...")
                    self.doOK("nphm", variogrammodel="exp")
                    self.writeInLog("Finished Kriging; NPHM; exponential variogram model")
                else:
                    self.writeInLog("ERROR: Could not find Layer: " + self.Layer)
                    raise AssertionError("Layer nicht gefunden")

            if "gau" in self.VarModel:
                if self.Layer == "foF2" or self.Layer == "NmF2":
                    self.writeInLog("Start Kriging; NPDM; gaussian variogram model...")
                    self.doOK("npdm", variogrammodel="gau")
                    self.writeInLog("Finished Kriging; NPDM; gaussian variogram model")
                elif self.Layer == "hmF2":
                    self.writeInLog("Start Kriging; NPHM; gaussian variogram model...")
                    self.doOK("nphm", variogrammodel="gau")
                    self.writeInLog("Finished Kriging; NPHM; gaussian variogram model")
                else:
                    self.writeInLog("ERROR: Could not find Layer: " + self.Layer)
                    raise AssertionError("Layer nicht gefunden")


            if "pow" in self.VarModel:
                if self.Layer == "foF2" or self.Layer == "NmF2":
                    self.writeInLog("Start Kriging; NPDM; power variogram model...")
                    self.doOK("npdm", variogrammodel="pow")
                    self.writeInLog("Finished Kriging; NPDM; power variogram model")
                elif self.Layer == "hmF2":
                    self.writeInLog("Start Kriging; NPHM; power variogram model...")
                    self.doOK("nphm", variogrammodel="pow")
                    self.writeInLog("Finished Kriging; NPHM; power variogram model")
                else:
                    self.writeInLog("ERROR: Could not find Layer: " + self.Layer)
                    raise AssertionError("Layer nicht gefunden")

        self.writeInLog("Finished calculation for: " + self.Layer + "\n--------------------")

    def getMinMaxBoundary(self, dict_lower, dict_upper, moy, station):
        '''
        This function searches for the maximal and minimal usable Data for hmF2 and NmF2. Reason: To find Outlyer.
        :param dict_lower: Dictionary from filter_option4.py
        :param dict_upper: Dictionary from filter_option4.py
        :param moy: Minute of the Year
        :param station: Station
        :return:
        '''

        interp_lower = dict_lower[station]
        interp_upper = dict_upper[station]
        data_bounds_lower = interp_lower(moy)
        data_bounds_upper = interp_upper(moy)

        if "NmF2" in self.Layer: return np.abs(data_bounds_lower) * 1e12, data_bounds_upper * 1e12
        else: return data_bounds_lower, data_bounds_upper

    def readData(self):
        '''
        This functions reads the hmF2 and NmF2 data. The Values and the coordinates will be stored in a "MeasurementStation" - object.
        :return:
        '''

        if self.Layer == "NmF2": layer = "foF2"
        elif self.Layer == "hmF2": layer = "hmF2"
        else: layer = "foF2"

        merge.param = layer
        if layer == "foF2":
            merge.dlpath_spidr = self.PathSpidr_NmF2
            merge.dlpath_all = self.PathDataAll_NmF2
            merge.dlpath_didb = self.PathDidb_NmF2
            merge.dlpath_ngdc = self.PathNgdc_NmF2
        elif layer == "hmF2":
            merge.dlpath_spidr = self.PathSpidr_hmF2
            merge.dlpath_all = self.PathDataAll_hmF2
            merge.dlpath_didb = self.PathDidb_hmF2
            merge.dlpath_ngdc = self.PathNgdc_hmF2
        else:
            self.writeInLog("ERROR: Could not find Layer: " + self.Layer)
            raise AssertionError("Bad layer input!")

        #merge.merge()
        allData = merge.return_dict()
        if not allData:
            self.writeInLog("ERROR: No Data found for " + self.Layer)
            raise AssertionError("No Data found for " + self.Layer)

        if "NmF2" in self.Layer:
            for station in allData.keys():
                values = allData[station]
                for dat in allData[station].keys():
                    if isinstance(values[dat], float):
                        values[dat] = 1.24e10*(values[dat])**2*1e-12

        #Filter
        FilterConfig.doy_start = 8
        FilterConfig.doy_end = 32
        if "hmF2" in self.Layer: FilterConfig.path_input = self.PathDataAll_hmF2
        else: FilterConfig.path_input = self.PathDataAll_NmF2
        FilterConfig.year_start = 2011
        lowerBound, upperBound = Filter.FilterOption4().filter4(self.Layer, allData)

        stations = allData.keys()
        self.Year = int(allData[stations[0]].keys()[0][:4])
        startdom, startmonth = self.getDayOfMonthandMonth(self.Startdoy)

        self.Month = startmonth

        miss_station = []

        #if(self.Layer == "foF2" or self.Layer == "NmF2"):
        #    maxValue = (FilterConfig.maxNmF2*10**10)
        #    minValue = (FilterConfig.minNmF2*10**10)
        #elif(self.Layer == "hmF2"):
        #    maxValue = FilterConfig.maxhmF2
        #    minValue = FilterConfig.minhmF2

        for station in stations:
            values = allData[station]
            dates = values.keys()           #Get Date+Time
            #Some stations have no "Meta" Information
            if not allData[station]["Meta"]:
                miss_station.append(station)
                continue
            isStation = False
            for date in dates:
                if date == "Meta": continue
                if len(date) != 16:
                    #print "WARNING: Datum: '" + date + "' is not the excpected dateformat (YYYY-MM-DD HH:MM)"
                    #self.writeInLog("WARNING: Datum: '" + date + "' is not the excpected dateformat (YYYY-MM-DD HH:MM)")
                    continue
                if "NmF2" in self.Layer: values[date] *= 1e12
                doy = self.getDayOfYear(int(date[8:-6]), int(date[5:-9]), int(date[:-12]))
                hour = int(date[11:-3]) + self.convertMinInHour(int(date[14:]))            #get the hour out of the datestring

                moy = doy * 24 * 60 + hour * 60
                minValue, maxValue = self.getMinMaxBoundary(lowerBound, upperBound, moy, station)

                localtimeMin = self.Hour - (self.convertMinInHour(self.TimeIntervall / 2))
                localtimeMax = self.Hour + (self.convertMinInHour(self.TimeIntervall / 2))
                ''' only neccessary, if the day changes in the timerange
                if localtimeMin < 0 and localtimeMax < 0:
                    localtimeMin += 24
                    localtimeMax += 24
                    doy -= 1
                elif localtimeMin >= 24 and localtimeMax >= 24:
                    localtimeMin -= 24
                    localtimeMax -= 24
                    doy += 1
                elif localtimeMin < 0:
                    localtimeMin += 24
                    localtimeMax = 24
                    doy -= 1
                    if localtimeMin <= hour < localtimeMax and self.Startdoy <= doy <= self.Enddoy:

                        if(minValue < values[date] < maxValue):
                            isStation = True
                            self.AllStations.addStation(allData[station]["Meta"]["Lat"], allData[station]["Meta"]["Lon"], values[date], station)
                    continue
                elif localtimeMax >= 24:
                    localtimeMax -= 24
                    localtimeMin = 0
                    doy += 1
                    if localtimeMin <= hour < localtimeMax and self.Startdoy == doy:
                        if(minValue < values[date] < maxValue):
                            isStation = True
                            self.AllStations.addStation(allData[station]["Meta"]["Lat"], allData[station]["Meta"]["Lon"], values[date], station)
                    continue
                '''
                if localtimeMin <= hour < localtimeMax and self.Startdoy == doy:
                    if(minValue < values[date] < maxValue):
                        isStation = True
                        self.AllStations.addStation(allData[station]["Meta"]["Lat"], allData[station]["Meta"]["Lon"], values[date], station)

            if isStation:
                meanOfOneStation = self.AllStations.getMeanOneStation(allData[station]["Meta"]["Lat"], allData[station]["Meta"]["Lon"])
                self.UniqueStations.addStation(allData[station]["Meta"]["Lat"], allData[station]["Meta"]["Lon"], meanOfOneStation, station)

        if self.UniqueStations.getCount() == 0:
            print "No Data found at " + str(self.Startdoy) + "_" + str(self.Time)
            #self.writeInLog("ERROR: No data in the timerange found")
            raise AssertionError("No data in the timerange found.")

        #stations with an error (missing Lat/Long value)
        if miss_station:
            print "WARNING: For this station(s) the metadata, in particular the latitude and longitude are missing: "
            print miss_station
            missing = ""
            for miss in miss_station:
                missing += str(miss) + "; "
            self.writeInLog("WARNING: For this station(s) the metadata, in particular the latitude and longitude are missing: " + missing)

    def generateValidationFile(self, filename):
        '''
        This function generates the validation file with the reference stations.
        :param filename: Filename
        :return:
        '''
        if not os.path.isfile(filename):
            rfn = open(filename, 'w')
            for i in range(len(self.RefStations)):
                rfn.write(self.RefStations[i] + "\n")
            rfn.close()

    def doOK(self, model, variogrammodel):
        '''
        This functions initialized the kriging calculation (inkl. Variograms) and calls the plotting methods. The kriging prediction
        and the variance prediction will be plotted

        :param model: Which model was used for the residuals. In this case NeQuick, NPDM, NPHM. This information is important for
            the filenames
        :param values: "MeasurementStation" - object, with unique stations
        :param variogrammodel: supported Variogrammmodels "exp", "sph" and "gau"
        :return:
        '''

        ordinarykriging = myOK.OrdinaryKriging(self.UniqueStations, variogrammodel, self.Outputfolder, model, self.DoValidation,
                                               self.F107, self.Hour, self.Startdoy, self.Month, self.OutputfolderLog, self.LogFileName, resolution=self.Resolution)
        ordinarykriging.calcVariogramParams(lag=self.LagDistance)
        predval, prederr = ordinarykriging.myKriging()

        if self.Layer == "NmF2":
            addHeader = "\n#Unit: electron*meter^(-3)"
            header = "#Layer: " + self.Layer + "\n#Variogrammodel: " + variogrammodel + "\n#Lag: " + str(self.LagDistance) \
                             + "\n#Resolution: " + str(self.Resolution) + " Grad" + addHeader
            filename = self.Outputfolder + "NmF2_predictionMatrix" + "_" + self.VarModel + "_" + self.BackModel + ".dat"
            np.savetxt(filename, predval, header=header)
            self.writeInListofResults(filename)
            filename = self.Outputfolder + "NmF2_predictionErrorMatrix" + "_" + self.VarModel + "_" + self.BackModel + ".dat"
            np.savetxt(filename, prederr, header=header)
            self.writeInListofResults(filename)
        else:
            addHeader = "\n#Unit: km"
            header = "#Layer: " + self.Layer + "\n#Variogrammodel: " + variogrammodel + "\n#Lag: " + str(self.LagDistance) \
                             + "\n#Resolution: " + str(self.Resolution) + " Grad" + addHeader
            filename = self.Outputfolder + "hmF2_predictionMatrix" + "_" + self.VarModel + "_" + self.BackModel + ".dat"
            np.savetxt(filename, predval, header=header)
            self.writeInListofResults(filename)
            filename = self.Outputfolder + "hmF2_predictionErrorMatrix" + "_" + self.VarModel + "_" + self.BackModel + ".dat"
            np.savetxt(filename, prederr, header=header)
            self.writeInListofResults(filename)


        #Not needed anymore... but it may be helpful in the future. Comparison between two estimations. Use carefully
        if self.DoValidation:
            if not self.PathToCompFileNmF2 == "" and self.Layer == "NmF2":
                if not os.path.isfile(self.PathToCompFileNmF2):
                    self.writeInLog("Could not find the compare file (NmF2)! Comparison will be skipped.")
                    print "Could not find the compare file (NmF2)! Comparison will be skipped."
                else:
                    self.writeInLog("Started validation for: " + self.Layer + "; " + variogrammodel + "; " + self.BackModel + "...")

                    dovalidation = KrigeVal.KrigeValidation(predval, header, self.Outputfolder, self.Layer, variogrammodel,
                                                            self.PathToCompFile, model)
                    dovalidation.doValidation()

            if not self.PathToCompFilehmF2 == "" and self.Layer == "hmF2":
                if not os.path.isfile(self.PathToCompFilehmF2):
                    self.writeInLog("Could not find the compare file (hmF2)! Comparison will be skipped.")
                    print "Could not find the compare file (hmF2)! Comparison will be skipped."
                else:
                    self.writeInLog("Started validation for: " + self.Layer + "; " + variogrammodel + "; " + self.BackModel + "...")

                    dovalidation = KrigeVal.KrigeValidation(predval, header, self.Outputfolder, self.Layer, variogrammodel,
                                                            self.PathToCompFile, model)
                    dovalidation.doValidation()
                    self.writeInLog("Finished validation for: " + self.Layer + "; " + variogrammodel + "; " + self.BackModel)


        ordinarykriging.plotKrigePrediction(predval, self.Layer + "_Kriging prediction_" + model + " " + variogrammodel)
        self.writeInListofResults(self.Layer + "_Kriging prediction_" + model + " " + variogrammodel + ".png")
        ordinarykriging.plotPredError(prederr, self.Layer + "_Variance Prediction_" + model + " " + variogrammodel)
        self.writeInListofResults(self.Layer + "_Kriging predictionError_" + model + " " + variogrammodel + ".png")

    def plotValueWithRespectToCoords(self, coordsUnique, coords_read, coordtyp):
        '''
        This function plots the value with respect to the coordtyp.
        :param coordsUnique:  list of unique coordinates of all stations. Only lat or lon
        :param coords_read: "MeasurementStation" - object with all values for the stations (not unique)
        :param coordtyp: "Latitude" or "Longitude" depended on the two other parameter
        :return:
        '''
        zipped = sorted(zip(coordsUnique, self.UniqueStations.getAllMeasurement()))
        coords, valueinlist = zip(*zipped)
        plt.figure()
        plt.plot(coords_read, self.AllStations.getAllMeasurement(), 'ro')
        plt.plot(list(coords), list(valueinlist))
        if coordtyp == "Latitude": plt.axis([-90, 90, 0, (max(valueinlist) + 5)])
        elif coordtyp == "Longitude": plt.axis([-180, 180, 0, (max(valueinlist) + 5)])
        plt.xlabel(coordtyp)
        plt.ylabel("measurement value")
        if coordtyp == "Latitude":
            plt.savefig(self.Outputfolder + self.Layer + "_MeasurementWithRespectToLat.png")
            self.writeInListofResults(self.Layer + "_MeasurementWithRespectToLat.png")
        else:
            plt.savefig(self.Outputfolder + self.Layer + "_MeasurementWithRespectToLon.png")
            self.writeInListofResults(self.Layer + "_MeasurementWithRespectToLon.png")
        plt.close()

    def plotHistogram(self):
        '''
        This function plots a histogram of the measurement values.
        :return:
        '''
        plt.figure()
        plt.hist(self.AllStations.getAllMeasurement())
        plt.xlabel("Measurement value")
        plt.ylabel("frequency")
        plt.savefig(self.Outputfolder + self.Layer + "_histogramMeasurement.png")
        self.writeInListofResults(self.Layer + "_histogramMeasurement.png")
        plt.close()

    def plothistTemp(self, values):
        plt.figure()
        plt.hist(values)
        plt.xlabel("Residuen")
        plt.ylabel("Haeufigkeit")
        plt.savefig(self.Outputfolder + "AAAAAAA_histogramResiduen.png")
        self.writeInListofResults(self.Layer + "_histogramMeasurement.png")
        plt.close()

    def plotNeQuickOneStation(self):
        '''
        This function plots the NeQuick values of one station.
        :return:
        '''

        oneStation = {
        'height': np.asarray(self.AltList),
        'lat': np.ones(len(self.AltList)) * self.UniqueStations.getLat(1),
        'lon': np.ones(len(self.AltList)) * self.UniqueStations.getLon(1),
        'month':self.Month,
        'f10p7': np.asarray(self.F107),
        'utcHour': self.Hour
        }

        wrapper = NeQuick.NeQuickWrapperVectorInput()
        wrapper.calcFromModel(oneStation)

        md = wrapper.modelData.values()[0]

        plt.figure()
        plt.plot(md, self.AltList)
        plt.ylabel("Altitude (km)")
        plt.xlabel("Electron content")
        plt.title("Station Lat: %1.2f Lon: %1.2f; " % (self.UniqueStations.getLat(1), self.UniqueStations.getLon(1))
                  + self.Time +"Uhr UTC")
        plt.savefig(self.Outputfolder + self.Layer + "_NeQuickOneStation.png")
        plt.close()
        self.writeInListofResults(self.Layer + "_NeQuickOneStation.png")

    def plotNPDMorNPHMOneStation(self):
        '''
        This function plots the NPDM or NPHM model of one station
        :return:
        '''
        allDoys = []
        i = self.Startdoy
        while i < self.Enddoy + 1:
            allDoys.append(i)
            i += 1

        allDoys = np.array(np.asarray(allDoys), float)
        sameLat = np.ones(len(allDoys)) * self.UniqueStations.getLat(1)
        sameLon = np.ones(len(allDoys)) * self.UniqueStations.getLon(1)
        F10p7_sfu = np.asarray(self.F107)
        UT_hrs = self.Hour

        if self.Layer == "foF2" or self.Layer == "NmF2":
            modelObj = NEDM.model(sameLat, sameLon, UT_hrs, allDoys, F10p7_sfu)
            modelresult = modelObj.NPDM()
            modelresult *= 10**12

        else:
            modelObj = NEDM.model(sameLat, sameLon, UT_hrs, allDoys, F10p7_sfu)
            modelresult = modelObj.NPHM()

        plt.figure()
        plt.ylim(min(modelresult)-1, max(modelresult)+1)
        plt.plot(allDoys, modelresult, 'ro')
        plt.xlabel("DOY")
        plt.title("Station Lat: %1.2f Lon: %1.2f; " % (self.UniqueStations.getLat(1), self.UniqueStations.getLon(1))
                  + self.Time +"Uhr UTC")

        if self.Layer == "NmF2" or self.Layer == "foF2":
            plt.ylabel("Electron content")
            plt.savefig(self.Outputfolder + self.Layer + "_NPDMOneStation.png")
            self.writeInListofResults(self.Layer + "_NPDMOneStation.png")
        elif self.Layer == "hmF2":
            plt.ylabel("Altitude (km)")
            plt.savefig(self.Outputfolder + self.Layer + "_NPHMOneStation.png")
            self.writeInListofResults(self.Layer + "_NPHMOneStation.png")
        plt.close()

    def plotNeQuickDiffMeasurement(self):
        '''
        This function plots the difference of the read values and the values of the NeQuick model with respect
        to the latitude and longitude.
        :return: Difference between the measurement value and the NeQuick value
        '''
        maxima = []
        for i in range(0, self.UniqueStations.getCount()):
            oneStation = {
            'height': np.asarray(self.AltList),
            'lat':  np.ones(len(self.AltList)) * self.UniqueStations.getLat(i),
            'lon': np.ones(len(self.AltList)) * self.UniqueStations.getLon(i),
            'month':self.Month,
            'f10p7':np.asarray(self.F107),
            'utcHour': self.Hour
            }
            wrapper = NeQuick.NeQuickWrapperVectorInput()
            wrapper.calcFromModel(oneStation)
            allValues = wrapper.modelData.values()[0]
            maximumValue = np.max(allValues)
            if self.Layer == "foF2" or self.Layer == "NmF2":
                maxima.append(maximumValue)
            else:
                for i in range(len(allValues)):
                    if allValues[i] == maximumValue:
                        maxima.append(self.AltList[i])

        #Maximalwert aus dem Modell von den Messwerten abziehen
        diffnequick = self.UniqueStations.diffModel(maxima)
        diffrelative = self.UniqueStations.diffModelRelative(maxima)
        self.plotDiff(self.UniqueStations.getAllLon(), diffrelative, self.Layer + "_DiffMeasurementNeQuickLon.png", "Longitude",
                      "Difference between Measurement and NeQuick")

        self.plotDiff(self.UniqueStations.getAllLat(), diffrelative, self.Layer + "_DiffMeasurementNeQuickLat.png", "Latitude",
                      "Difference between Measurement and NeQuick")


        return diffnequick

    def plotDiff(self, coordsUnique, diff, filename, xlab, ylab):
        '''
        This function plots the relative
        :param coordsUnique: list of unique coordinates of all stations. Only lat or lon
        :param diff: difference between the measurement value and the model value
        :param filename: Filename of the output picture
        :param xlab: name of the x-axis
        :param ylab: name of the y-axis
        :return:
        '''
        zipped = sorted(zip(coordsUnique, diff))
        coords, valueinlist = zip(*zipped)
        plt.figure()
        plt.plot(coordsUnique, diff, 'ro')
        plt.plot(coords, valueinlist)
        plt.xlabel(xlab)
        plt.ylabel(ylab)
        plt.savefig(self.Outputfolder + filename)
        plt.close()
        self.writeInListofResults(filename)

    def plotStations(self):
        '''
        This function plots the measurement stations on a map.
        :return:
        '''
        plt.figure()
        map = Basemap(projection='cyl', lat_0=0, lon_0=0, resolution='l', ellps="WGS84", area_thresh=1000.0, llcrnrlon=-180,
                      llcrnrlat=-90, urcrnrlon=180, urcrnrlat=90)

        map.drawcoastlines()
        map.drawcountries()
        map.drawmapboundary()
        parallels = np.array((60, 30, 0, -30, -60))
        map.drawparallels(parallels, labels=[1, 0, 0, 0], fontsize=12)
        meridians = np.array((90, 0, -90))
        map.drawmeridians(meridians, labels=[0, 0, 0, 1], fontsize=12)

        map.scatter(self.UniqueStations.getAllLon(), self.UniqueStations.getAllLat(), marker=u'^', color='r', linewidths=2)

        plt.title("Interpolation area with measurement stations")
        filename = self.Layer + "_Interpolationarea.png"
        plt.savefig(self.Outputfolder + filename)
        plt.close()
        self.writeInListofResults(filename)

    def plotNPDMDorNPHMDiffMeasurement(self):
        '''
        This function reads the model value for NPDM and NPHM. The model, which should be read, depends on the Layer (hmF2 or NmF2)
        :return: Difference between measurement value and model value
        '''
        allDoys = []
        i = self.Startdoy
        while i < self.Enddoy + 1:
            allDoys.append(i)
            i += 1
        doy = np.array(np.asarray(allDoys), float)

        meanAlt = []
        F10p7_sfu = np.array(self.F107, float)                                              #solar radio flux in flux units Vector
        for i in range(0, self.UniqueStations.getCount()):

            lat_deg = np.ones(len(doy)) * self.UniqueStations.getLat(i)
            lon_deg = np.ones(len(doy)) * self.UniqueStations.getLon(i)
            UT_hrs = np.array([self.Hour], float)
            modelObj = NEDM.model(lat_deg, lon_deg, UT_hrs, doy, F10p7_sfu)
            if self.Layer == "NmF2":
                npdm = modelObj.NPDM()
                meanAlt.append(float(npdm * (10**12)))
            else:
                nphm = modelObj.NPHM()
                meanAlt.append(float(nphm))

        diff = self.UniqueStations.diffModel(meanAlt)
        diffrelative = self.UniqueStations.diffModelRelative(meanAlt)
        self.plotDiff(self.UniqueStations.getAllLon(), diffrelative, self.Layer + "_DiffMeasurementNPDMLon.png", "Longitude", "Difference between Measurement and NPDM")

        self.plotDiff(self.UniqueStations.getAllLat(), diffrelative, self.Layer + "_DiffMeasurementNPDMLat.png", "Latitude", "Difference between Measurement and NPDM")

        return diff

    def getf107Values(self):
        '''
        This function gets the F10.7 values for the requested day of years in the config file
        :return:
        '''
        try:
            for i in range(0, self.Enddoy + 1 - self.Startdoy):
                dom, month = self.getDayOfMonthandMonth(self.Startdoy + i)
                f107 = KPReader.KPFileReader(self.PathF107).getF10p7(self.Year, month, dom)
                self.F107.append(float(f107[0]))
        except:
            self.writeInLog("ERROR: KPFileReader had an Error! Solution: Check if the Path is correct, check if the correct files are available "
                                 "for the year, month and day you requested")
            raise AssertionError("ERROR: KPFileReader had an Error! Solution: Check if the Path is correct, check if the correct files are available "
                                 "for the year, month and day you requested")

    def getDayOfMonthandMonth(self, doy):
        '''
        This function calculates the month and the day of the month from a day of year - value
        :param doy: day of year
        :return: day of month and month
        '''
        if doy == 1:
            mydate = str(datetime.datetime(self.Year, 1, 1) + datetime.timedelta(doy))
        else:
            mydate = str(datetime.datetime(self.Year, 1, 1) + datetime.timedelta(doy - 1))
        month = int(mydate[5: -12])
        dom = int(mydate[8: -9])
        return dom, month

    def getDayOfYear(self, dom, month, year):
        '''
        This function calculates the day of year.
        :param dom: day of month
        :param month: month
        :param year: year
        :return: day of year
        '''
        if year % 4 == 0: leapyear = 1
        else: leapyear = 0
        if month == 1: return dom
        elif month == 2: return 31 + dom
        elif month == 3: return 59 + leapyear + dom
        elif month == 4: return 90 + leapyear + dom
        elif month == 5: return 120 + leapyear + dom
        elif month == 6: return 151 + leapyear + dom
        elif month == 7: return 181 + leapyear + dom
        elif month == 8: return 212 + leapyear + dom
        elif month == 9: return 243 + leapyear + dom
        elif month == 10: return 273 + leapyear + dom
        elif month == 11: return 304 + leapyear + dom
        elif month == 12: return 334 + leapyear + dom
        else:
            self.writeInLog("ERROR: Something went horrible wrong! Could not find the month:" + str(month))
            raise AssertionError("Something went horrible wrong! Could not find the month:" + str(month))

    def convertMinInHour(self, min):
        '''
        This function converts minutes to hour
        :param min: minutes
        :return: hour
        '''
        return (1.0 / 60.0) * int(min)

    def writeInLog(self, message):
        '''
        This function writes all massages in a log file.
        :param message: message to write
        :return:
        '''
        fobj = open(self.OutputfolderLog + self.LogFileName, "a")
        fobj.writelines(message + "\n")
        fobj.close()

    def writeInListofResults(self, filename):
        '''
        This function will write all result filenames into one file.
        :param message: message to write
        :return:
        '''
        fobj = open(self.Outputfolder + "ListOfResults.dat", "a")
        fobj.writelines(filename + "\n")
        fobj.close()

    def plotHistMeasurementEstimation(self):
        '''
        This function organized the plot of the validation file as a histogram.
        :return:
        '''
        temparr = np.asarray(([0, 0], [0, 0]))
        kv = KrigeVal.KrigeValidation(temparr, 0, 0, 0, 0, 0, 0)

        filename = self.OutputfolderLog + "NmF2_npdmexpResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "NmF2_npdmexpResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals", "Samples")
        filename = self.OutputfolderLog + "NmF2_npdmgauResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "NmF2_npdmgauResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals", "Samples")
        filename = self.OutputfolderLog + "NmF2_npdmsphResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "NmF2_npdmsphResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals", "Samples")
        filename = self.OutputfolderLog + "NmF2_npdmpowResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "NmF2_npdmpowResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals", "Samples")

        filename = self.OutputfolderLog + "NmF2_nequickexpResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "NmF2_nequickexpResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals", "Samples")
        filename = self.OutputfolderLog + "NmF2_nequickgauResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "NmF2_nequickgauResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals", "Samples")
        filename = self.OutputfolderLog + "NmF2_nequicksphResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "NmF2_nequicksphResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals", "Samples")
        filename = self.OutputfolderLog + "NmF2_nequickpowResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "NmF2_nequickpowResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "NmF2 Residuals", "Samples")


        filename = self.OutputfolderLog + "hmF2_nphmexpResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "hmF2_nphmexpResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals", "Samples")
        filename = self.OutputfolderLog + "hmF2_nphmsphResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "hmF2_nphmsphResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals", "Samples")
        filename = self.OutputfolderLog + "hmF2_nphmgauResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "hmF2_nphmgauResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals", "Samples")
        filename = self.OutputfolderLog + "hmF2_nphmpowResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "hmF2_nphmpowResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals", "Samples")

        filename = self.OutputfolderLog + "hmF2_nequickexpResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "hmF2_nequickexpResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals", "Samples")
        filename = self.OutputfolderLog + "hmF2_nequicksphResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "hmF2_nequicksphResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals", "Samples")
        filename = self.OutputfolderLog + "hmF2_nequickgauResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "hmF2_nequickgauResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals", "Samples")
        filename = self.OutputfolderLog + "hmF2_nequickpowResidualsRefRel.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals in %", "Samples")
        filename = self.OutputfolderLog + "hmF2_nequickpowResidualsRefAbs.dat"
        if os.path.isfile(filename): kv.plotHistMeasurementEstimation(filename, "hmF2 Residuals", "Samples")

    #Model Validation stuff! no need for the future
    def testModelNPDMNPDHM(self, lat, lon):

        allDoys = np.array(np.asarray(np.asarray([self.Startdoy])), float)
        sameLat = np.ones(len(allDoys)) * lat
        sameLon = np.ones(len(allDoys)) * lon
        F10p7_sfu = np.asarray(self.F107)[0]
        UT_hrs = self.Hour

        if self.Layer == "NmF2":
            modelObj = NEDM.model(sameLat, sameLon, UT_hrs, allDoys, F10p7_sfu)
            modelresult = modelObj.NPDM()
            modelresult = float(modelresult) * (10**12)
        else:
            modelObj = NEDM.model(sameLat, sameLon, UT_hrs, allDoys, F10p7_sfu)
            modelresult = modelObj.NPHM()

        return modelresult

    def testModelNeQuick(self, lat, lon):
        oneStation = {
        'height': np.asarray(self.AltList),
        'lat':  np.ones(len(self.AltList)) * lat,
        'lon': np.ones(len(self.AltList)) * lon,
        'month':self.Month,
        'f10p7':np.asarray(self.F107),
        'utcHour': self.Hour
        }
        wrapper = NeQuick.NeQuickWrapperVectorInput()
        wrapper.calcFromModel(oneStation)
        allValues = wrapper.modelData.values()[0]
        maximumValue = np.max(allValues)
        if self.Layer == "foF2" or self.Layer == "NmF2":
            return maximumValue
        else:
            for i in range(len(allValues)):
                if allValues[i] == maximumValue:
                    return self.AltList[i]

    def readMatrix(self):
        '''
        if self.Layer == "NmF2":
            fobj = open("/home/linux/Development/projects/TBojeUtils/NmF2_NPHM_2011-011-09-30.dat", "r")
        else:
            fobj = open("/home/linux/Development/projects/TBojeUtils/hmF2_NPHM_2011-011-09-30.dat", "r")

        readlines = []
        skipheader = 0
        for line in fobj:
            skipheader += 1                 #skip the header
            if skipheader == 1: continue
            readlines.append(line)
        fobj.close()
        '''
        resolution = 2.5
        lons = []
        lats = []

        i = -90
        while i <= 90:
            lats.append(i)
            i += resolution

        i = -180
        while i <= 180:
            if i == 0:
                lons.append(i + 0.0000001)
                i += resolution
                continue
            lons.append(i)
            i += resolution
        '''
        readlistDat = np.zeros((len(lats), len(lons)))
        row = len(readlines)
        column = len(readlines[0].split())
        for i in range(0, row):
            splittedRow = readlines[i].split()
            for j in range(0, column):
                if self.Layer == "NmF2":
                    readlistDat[i][j] = float(splittedRow[j])
                else: readlistDat[i][j] = float(splittedRow[j])
        '''
        readlistModel = np.zeros((len(lats), len(lons)))
        if self.BackModel == "nphm" or self.BackModel == "npdm":
            counterLats = 0
            for i in range(len(lats)):
                counterLons = 0
                for j in range(len(lons)):
                    readlistModel[i][j] = self.testModelNPDMNPDHM(lats[i], lons[j])
                    counterLons += 1
                counterLats += 1
        elif self.BackModel == "nequick":
            counterLats = 0
            for i in range(0, len(lats)):
                counterLons = 0
                for j in range(0, len(lons)):
                    readlistModel[counterLats][counterLons] = self.testModelNeQuick(lats[counterLats], lons[counterLons])
                    counterLons += 1
                counterLats += 1
        else:
            raise AssertionError("OHA")

        #differenzen = np.subtract(readlistModel, readlistDat)
        #print len(readlistModel), len(differenzen), len(differenzen[0])
        differenzen = np.zeros((len(readlistModel), len(readlistModel[0])))
        for i in range(len(lats)):
            for j in range(len(lons)):
                differenzen[i][j] = readlistModel[i][j]


        #np.savetxt("AAABerechneteWerteNmF2.txt", readlistModel)

        #np.savetxt("AAAPythonNPDMDifferenzen.txt", differenzen)

        plt.figure()
        map = Basemap(projection='cyl', lat_0=0, lon_0=0, resolution='l', ellps="WGS84", area_thresh=1000.0, llcrnrlon=-180,
                      llcrnrlat=-90, urcrnrlon=180, urcrnrlat=90)
        map.drawcoastlines()
        map.drawcountries()
        map.drawmapboundary()
        #image = map.imshow(differenzen, cmap="ocean", origin='lower')
        parallels = np.array((60, 30, 0, -30, -60))
        map.drawparallels(parallels, labels=[1, 0, 0, 0], fontsize=12)
        meridians = np.array((90, 0, -90))
        map.drawmeridians(meridians, labels=[0, 0, 0, 1], fontsize=12)
        if self.Layer == "hmF2":
            colorbarscaling = np.asarray((150, 200, 250, 300, 350, 400, 450))
            vmin = 150
            vmax = 450
        else:
            colorbarscaling = np.asarray((0.0, 0.2, 0.4, 0.6, 0.8, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0)) * 10**12
            vmin = 0
            vmax = 2.0 * 10**12

        #plt.colorbar(image, orientation='horizontal', ticks=np.asarray(colorbarscaling))
        image = map.imshow(differenzen, cmap="jet", origin='lower', norm=mc.Normalize(vmin=vmin, vmax=vmax))
        legend = map.colorbar(image, location='bottom', pad="10%", ticks=colorbarscaling)
        legend.set_clim(vmin=vmin, vmax=vmax)

        #plot the measurement points
        colorMap = image.get_cmap()
        cNorm = matplotlib.colors.Normalize()
        matplotlib.cm.ScalarMappable(norm=cNorm, cmap=colorMap)
        plt.title(self.Layer + " " + self.BackModel)
        plt.savefig(self.Outputfolder + "Differences_" + self.Layer + "_" + self.BackModel + ".png")

        #print differenzen

    def findMinMax(self, matrix):
        '''
        This functions finds the minimum and maximum value in a matrix
        :param matrix: Matrix
        :return: minumum value, maximum value
        '''
        minval = matrix[0][0]
        maxval = matrix[0][0]

        for i in range(len(matrix)):
            for j in range(len(matrix[0])):
                if minval > matrix[i][j]: minval = matrix[i][j]
                elif maxval < matrix[i][j]: maxval = matrix[i][j]
        return minval, maxval