import os
import sys

#-------------------------------------------------------------------------------
# Input Parameters
#-------------------------------------------------------------------------------

begin_dd = "08"
begin_mm = "01"
begin_yyyy = "2011"
end_dd = "01"
end_mm = "02"
end_yyyy = "2011"
iono_param = "hmF2"
dlpath = "/home/linux/IonoDL/hmF2"

#iono_station = "JR055" # for download_data_station() only

#-------------------------------------------------------------------------------
# Configuration
#
# (Don't change unless you know what you're doing)
#-------------------------------------------------------------------------------

begin_yyyymmdd = begin_yyyy + begin_mm + begin_dd
end_yyyymmdd = end_yyyy + end_mm + end_dd

# All 49 possible ionospheric parameters
# in the order of SAO version 4.3 / ARTIST
available_parameters = [
"foF2", "foF1", "MD", "MUFD", "fmin", 
"foEs", "fminF", "fminE", "foE", "fxI", 
"hF", "hF2", "hE", "hEs", "hmE", 
"yE", "QF", "QE", "DownF", "DownE", 
"DownEs", "FF", "FE", "D", "fMUF", 
"hfMUF", "delta_foF2", "foEp", "fhF", "fhF2", 
"foF1p", "hmF2", "hmF1", "zhalfNm", "foF2p", 
"fminEs", "yF2", "yF1", "TEC", "scaleF2", 
"B0", "B1", "D1", "foEa", "hEa", 
"foP", "hP", "fbEs", "TypeEs"
]

# Exit program if parameter is not defined
if(iono_param not in available_parameters):
    sys.exit('Parameter "' + iono_param + '" not defined')

# Get unit of specified parameter
unit = ""
if(iono_param in [
"foF2", "foF1", "MUFD", "fmin", 
"foEs", "fminF", "fminE", "foE", "fxI", 
"FF", "FE", "fMUF", 
"delta_foF2", "foEp", "fhF", "fhF2", 
"foF1p", "foF2p", 
"fminEs", 
"foEa", 
"foP", "fbEs", 
]):
    unit = "MHz"
elif(iono_param in [
"hF", "hF2", "hE", "hEs", "hmE", 
"yE", "QF", "QE", "DownF", "DownE", 
"DownEs", "D", 
"hfMUF", 
"hmF2", "hmF1", "zhalfNm", 
"yF2", "yF1", "scaleF2", 
"B0", "hEa", 
"hP", 
]):
    unit = "km"
elif(iono_param in [
"MD", 
"B1", "D1", 
"TypeEs", 
]):
    unit = "-"
elif(iono_param in [
"TEC", 
]):
    unit = "10^16 m^-2"

# Create DL paths
dlpath_ngdc_raw = os.path.join(dlpath, "data_ngdc_raw")
dlpath_ngdc_dat = os.path.join(dlpath, "data_ngdc_dat")
dlpath_spidr = os.path.join(dlpath, "data_spidr")
dlpath_spidr_meta = os.path.join(dlpath, "data_spidr_meta")
dlpath_didb = os.path.join(dlpath, "data_didb")
dlpath_all = os.path.join(dlpath, "data_all")

if not (os.path.isdir(dlpath_ngdc_raw)):
    os.makedirs(dlpath_ngdc_raw)
if not (os.path.isdir(dlpath_ngdc_dat)):
    os.makedirs(dlpath_ngdc_dat)
if not (os.path.isdir(dlpath_spidr)):
    os.makedirs(dlpath_spidr)
if not (os.path.isdir(dlpath_spidr_meta)):
    os.makedirs(dlpath_spidr_meta)
if not (os.path.isdir(dlpath_didb)):
    os.makedirs(dlpath_didb)
if not (os.path.isdir(dlpath_all)):
    os.makedirs(dlpath_all)
