import csv

from bin.reconstruct.module import config
import os


#param = module.config.iono_param
#dlpath_spidr = module.config.dlpath_spidr
#dlpath_didb = module.config.dlpath_didb
#dlpath_ngdc = module.config.dlpath_ngdc_dat
#dlpath_all = module.config.dlpath_all

param = ""
dlpath_spidr = ""
dlpath_didb = ""
dlpath_ngdc = ""
dlpath_all = ""

dict_spidr = {}
dict_didb = {}
dict_ngdc = {}
dict_all = {}

dict_latlon_spidr = {}
dict_latlon_didb = {}
dict_latlon_ngdc = {}
dict_latlon_all = {}


def create_dict_spidr():
    """
    This function fills the dictionaries ``dict_spidr`` and ``dict_latlon_spidr`` with ionosonde data and metadata respectively.
    
    The data is read from all files in ``module.config.dlpath_spidr`` that contain data of the parameter configured in ``module.config.iono_param``.
    
    The metadata is read from the file ``latlonlist.csv`` in ``module.config.dlpath_spidr``, created with the function :py:func:`provider.spidr.download_list_latlon`.
    """
    list_files = [filename for filename in os.listdir(dlpath_spidr) if filename.startswith(param) and filename.endswith(".dat")]
    listlatlon = []
    lllexists = False
    if(os.path.isfile(os.path.join(dlpath_spidr, "latlonlist.csv"))):
        lllexists = True
        frlll = open(os.path.join(dlpath_spidr, "latlonlist.csv"), "r")
        for line in frlll:
            listlatlon.append(line)
        frlll.close()
    for datfile in list_files:
        station = datfile[len(param)+1:-4]
        # dict_spidr
        fr = open(os.path.join(dlpath_spidr, datfile), "r")
        dict_station = {}

        for row in fr:
            if not "#" in row:
                if row[17:].find(",") != -1:
                    value = float(row[17:((row[17:].find(",") + 17) - len(row))])
                else:
                    value = float(row[17:])
                time = row[0:16]
                dict_station[time] = value
        fr.close()
        dict_spidr[station] = dict_station
        # dict_latlon_spidr
        if(lllexists):
            station_line = next(line for line in listlatlon if station in line)
            for item in csv.reader([station_line]):
                if item[1] and item[2] and item[3]:
                    name = item[1]
                    lat = item[2]
                    lon = item[3]
                    dict_latlon_spidr[station] = {"Name" : name, "Lat" : lat, "Lon" : lon}

def create_dict_didb():
    """
    This function fills the dictionaries ``dict_didb`` and ``dict_latlon_didb`` with ionosonde data and metadata respectively.
    
    The data is read from all files in ``module.config.dlpath_didb`` that contain data of the parameter configured in ``module.config.iono_param``.
    
    The metadata is read from the file ``latlonlist.csv`` in ``module.config.dlpath_spidr``, created with the function :py:func:`provider.spidr.download_list_latlon`, or is read directly from the .dat files.
    """
    list_files = [filename for filename in os.listdir(dlpath_didb) if filename.startswith(param) and filename.endswith(".dat")]
    listlatlon = []
    lllexists = False
    if(os.path.isfile(os.path.join(dlpath_didb, "latlonlist.csv"))):
        lllexists = True
        frlll = open(os.path.join(dlpath_didb, "latlonlist.csv"), "r")
        for line in frlll:
            listlatlon.append(line)
        frlll.close()
    for datfile in list_files:
        station = datfile[len(param)+1:-4]
        # dict_didb
        fr = open(os.path.join(dlpath_didb, datfile), "r")
        dict_station = {}
        for row in fr:
            if not "#" in row:
                rowlist = row.split()
                time = rowlist[0][:10] + " " + rowlist[0][11:16]
                value = float(rowlist[2])
                dict_station[time] = value
        fr.close()
        dict_didb[station] = dict_station
        # dict_latlon_didb
        if(lllexists):
            station_line = next(line for line in listlatlon if station in line)
            for item in csv.reader([station_line]):
                if item[1] and item[2] and item[3]:
                    name = item[1]
                    lat = item[2]
                    lon = float(item[3])
                    if(lon>180):
                        lon -= 360
                    lon = str(lon)
                    dict_latlon_didb[station] = {"Name" : name, "Lat" : lat, "Lon" : lon}
        else:
            fr = open(os.path.join(dlpath_didb, datfile), "r")
            latlon_line =  next(line for line in fr if "Location" in line)
            for item in csv.reader([latlon_line], delimiter=" "):
                # Format: "# Location: GEO 00.0N 00.0E, URSI-Code URSIC STATION NAME"
                lat = item[3][:-1]
                lon = float(item[4][:-2])
                if(lon>180):
                    lon -= 360
                lon = str(lon)
                # ["STATION", "NAME"] to "Station Name"
                namelist = item[7:]
                name = " ".join(x.capitalize() for x in namelist)
                dict_latlon_didb[station] = {"Name" : name, "Lat" : lat, "Lon" : lon}
            fr.close()

def create_dict_ngdc():
    """
    This function fills the dictionaries ``dict_ngdc`` and ``dict_latlon_ngdc`` with ionosonde data and metadata respectively.
    
    The data is read from all files in ``module.config.dlpath_ngdc`` that contain data of the parameter configured in ``module.config.iono_param``.
    
    The metadata is read directly from the .dat files.
    """
    list_files = [filename for filename in os.listdir(dlpath_ngdc) if filename.startswith(param) and filename.endswith(".dat")]
    for datfile in list_files:
        station = datfile[len(param)+1:-4]
        # dict_ngdc
        fr = open(os.path.join(dlpath_ngdc, datfile), "r")
        dict_station = {}
        for row in fr:
            if not "#" in row:
                rowlist = row.split()
                time = rowlist[0] + " " + rowlist[1][:5]
                value = float(rowlist[2])
                dict_station[time] = value
            # dict_latlon_ngdc
            else:
                if "Lat" in row:
                    rowlist = row.split()
                    lat = rowlist[1][4:]
                    lon = float(rowlist[2][4:])
                    if(lon>180):
                        lon -= 360
                    lon = str(lon)
                    dict_latlon_ngdc[station] = {"Lat" : lat, "Lon" : lon}
                if "NAME" in row:
                    header = csv.reader([row])
                    for line in header:
                        namefield = next(item for item in line if "NAME" in item)
                        name = namefield.strip()[5:]
                        dict_latlon_ngdc[station]["Name"] = name
        fr.close()
        dict_ngdc[station] = dict_station

# Merge data from dlpaths into a single file per station in dlpath_all
def merge():
    """
    This function reads the files downloaded from the different providers and merges the dictionaries into one dictionary.
    
    The order in which the providers are processed is:
    
    #. SPIDR
    
    #. DIDBase
    
    #. FTP NGDC
    
    *Concerning different values at the same time:* This function overwrites a value already collected in the dictionary with a value of another provider, if the latter has more decimal places and rounds to the same value.
    
    A file named ``differences.txt`` is created to show completely different values between the providers.
    
    The differences listed in ``differences.txt`` are:
    
    * one value has more decimal places but doesn't round to the other value
    
    * equal decimal places, but different values
    
    Metadata is also collected from the different providers.
    
    In the end the complete dictionary is output to one .dat file per station in ``module.config.dlpath_all``. To get a Python dictionary of the data you can use :py:func:`return_dict`.
    """

    #param = _param
    #dlpath_spidr = _dlpath_spidr
    #dlpath_ngdc = _dlpath_ngdc
    #dlpath_didb = _dlpath_didb
    #dlpath_all = _dlpath_all


    #print "ALL - Merge Data"
    create_dict_spidr()
    #create_dict_didb()
    #create_dict_ngdc()
    
    '''DATA DICTIONARY'''
    
    # ADD SPIDR
    dict_all = dict_spidr.copy()
    
    # ADD DIDB
    # 3 possibilities: stations only in SPIDR, stations only in DIDB, intersection (spidr+didb)
    # 1: SPIDR-only stations already in dict_all
    # 2: add DIDB-only stations
    set_all = set(dict_all.keys())
    set_didb = set(dict_didb.keys())
    intersect_stations = set_all.intersection(set_didb)
    set_only_didb = set_didb - intersect_stations
    for station in set_only_didb:
        dict_all[station] = dict_didb[station]
    # 3: intersection
    for station in intersect_stations:
        # again 3 possibilities
        # 3.1: SPIDR-only time data already in dict_all
        # 3.2: add DIDB-only time data
        set_station_all = set(dict_all[station].keys())
        set_station_didb = set(dict_didb[station].keys())
        intersect_time = set_station_all.intersection(set_station_didb)
        set_station_only_didb = set_station_didb - intersect_time
        for time in set_station_only_didb:
            dict_all[station][time] = dict_didb[station][time]
        # 3.3: intersection
        for time in intersect_time:
            fw=open(os.path.join(dlpath_all, "differences.txt"), "a")
            value_all = dict_all[station][time]
            value_didb = dict_didb[station][time]
            if(value_all != value_didb):
                len_value_all = len(str(value_all).split(".")[1])
                len_value_didb = len(str(value_didb).split(".")[1])
                if(len_value_all > len_value_didb):
                    if not(round(value_all, 1)==value_didb):
                        fw.write(station + " " + time + " Previous: " + str(value_all) + " DIDB: " + str(value_didb) + "\n")
                elif(len_value_all < len_value_didb):
                    if(round(value_didb, 1)==value_all):
                        dict_all[station][time] = value_didb
                    else:
                        fw.write(station + " " + time + " Previous: " + str(value_all) + " DIDB: " + str(value_didb) + "\n")
                else:
                    fw.write(station + " " + time + " Previous: " + str(value_all) + " DIDB: " + str(value_didb) + "\n")
    
    # ADD NGDC
    # 2
    set_all = set(dict_all.keys())
    set_ngdc = set(dict_ngdc.keys())
    intersect_stations = set_all.intersection(set_ngdc)
    set_only_ngdc = set_ngdc - intersect_stations
    for station in set_only_ngdc:
        dict_all[station] = dict_ngdc[station]
    # 3
    for station in intersect_stations:
        # 3.2
        set_station_all = set(dict_all[station].keys())
        set_station_ngdc = set(dict_ngdc[station].keys())
        intersect_time = set_station_all.intersection(set_station_ngdc)
        set_station_only_ngdc = set_station_ngdc - intersect_time
        for time in set_station_only_ngdc:
            dict_all[station][time] = dict_ngdc[station][time]
        # 3.3
        for time in intersect_time:
            fw=open(os.path.join(dlpath_all, "differences.txt"), "a")
            value_all = dict_all[station][time]
            value_ngdc = dict_ngdc[station][time]
            if(value_all != value_ngdc):
                len_value_all = len(str(value_all).split(".")[1])
                len_value_ngdc = len(str(value_ngdc).split(".")[1])
                if(len_value_all > len_value_ngdc):
                    if not(round(value_all, 1)==value_ngdc):
                        fw.write(station + " " + time + " Previous: " + str(value_all) + " NGDC: " + str(value_ngdc) + "\n")
                elif(len_value_all < len_value_ngdc):
                    if(round(value_ngdc, 1)==value_all):
                        dict_all[station][time] = value_ngdc
                    else:
                        fw.write(station + " " + time + " Previous: " + str(value_all) + " NGDC: " + str(value_ngdc) + "\n")
                else:
                    fw.write(station + " " + time + " Previous: " + str(value_all) + " NGDC: " + str(value_ngdc) + "\n")
    
    '''METADATA'''
    
    # ADD SPIDR
    dict_latlon_all = dict_latlon_spidr.copy()
    # ADD DIDB
    # 2
    set_all = set(dict_latlon_all.keys())
    set_didb = set(dict_latlon_didb.keys())
    intersect_stations = set_all.intersection(set_didb)
    set_only_didb = set_didb - intersect_stations
    for station in set_only_didb:
        dict_latlon_all[station] = dict_latlon_didb[station]
    # 3
    for station in intersect_stations:
        pass # TODO: Intersection
    # ADD NGDC
    # 2
    set_all = set(dict_latlon_all.keys())
    set_ngdc = set(dict_latlon_ngdc.keys())
    intersect_stations = set_all.intersection(set_ngdc)
    set_only_ngdc = set_ngdc - intersect_stations
    for station in set_only_ngdc:
        dict_latlon_all[station] = dict_latlon_ngdc[station]
    # 3
    for station in intersect_stations:
        pass # TODO: Intersection
    
    '''WRITE DATA'''
    
    print("ALL - Write Merged Data")
    for station in dict_all:
        fw = open(os.path.join(dlpath_all, param + "_" + station + ".dat"), "w")
        if(station in dict_latlon_all):
            if("Name" in dict_latlon_all[station]):
                fw.write("# " + station + " " + dict_latlon_all[station]["Name"] + "\n")
            else:
                fw.write("# " + station + " " + station + "\n")
            fw.write("# Lat:" + dict_latlon_all[station]["Lat"] + " Lon:" + dict_latlon_all[station]["Lon"] + "\n")
        fw.write("# YYYY-MM-DD hh:mm " + param + " [" + config.unit + "]\n")
        for k,v in sorted(dict_all[station].items()):
            fw.write(k + " " + str(v) + "\n")
        fw.close

# Return dictionary for specified parameter with format dict["URSI-Code"]["YYYY-MM-DD hh:mm"] = Value
def return_dict():
    """
    This function reads all files in ``module.config.dlpath_all`` created with the function :py:func:`merge` that contain data of the parameter configured in ``module.config.iono_param`` and returns a dictionary with the data and metadata.
    
    :return: dictionary of ionosonde data, format ``dict["Station ID"]["YYYY-MM-DD hh:mm"] = float`` and for metadata ``dict["Station ID"]["Meta"]["Name"/"Lat"/"Lon"] = str/float/float``
    :rtype: dict
    """
    #print("ALL - Return Merged Data")

    dict_data = {}
    list_files = [filename for filename in os.listdir(dlpath_all) if filename.startswith(param)]
    for filename in list_files:
        fr = open(os.path.join(dlpath_all, filename), "r")
        filecontent = fr.readlines()
        fr.close()
        station = filename[len(param)+1:-4]
        dict_data[station] = {}
        dict_data[station]["Meta"] = {}
        for contentline in filecontent:
            if("#" in contentline):
                if(station in contentline):
                    line = contentline.split()
                    name = ' '.join(line[2:])
                    dict_data[station]["Meta"]["Name"] = name
                if("Lat" in contentline):
                    line = contentline.split()
                    lat = ""
                    lon = ""
                    if(len(line[1]) > 4):
                        lat = float(line[1][4:])
                        lon = float(line[2][4:])
                    dict_data[station]["Meta"]["Lat"] = lat
                    dict_data[station]["Meta"]["Lon"] = lon
            else:
                line = contentline.split()
                valuedate = line[0]
                valuetime = line[1]
                valuedatetime = valuedate + " " + valuetime
                value = float(line[2])
                if(value!=9999.0):
                    dict_data[station][valuedatetime] = value
    return dict_data

def main():
    pass

if __name__ == "__main__":
    main()
